package tsyliveaudiorecord.live_audio_record;


import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;

import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import android.os.Build;
import android.os.IBinder;
import android.util.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;

/** LiveAudioRecordPlugin */
public class VaisSpeechPlugin implements MethodCallHandler, PluginRegistry.RequestPermissionsResultListener {
    private static final String LOG_NAME = "AndroidAudioRecorder";
    private static final int PERMISSIONS_REQUEST_RECORD_AUDIO = 200;
    private static final byte RECORDER_BPP = 16; // we use 16bit
    private Registrar registrar;
    private int mSampleRate = 16000; // 16Khz
    private Boolean startForeground = false;
    private Boolean isWriteFile = false;
    private String mFilePath;
    private String mExtension;
    private int bufferSize = 1024;
    private FileOutputStream mFileOutputStream = null;
    private String mStatus = "unset";

    private long mDataSize = 0;
    private Result _result;
    private byte[] tempByteBuffer;
    Queue<HashMap<String, Object>> qe;
    Intent intent = null;
    AudioRecordService audioRecordService;


    /** Plugin registration. */
    public static void registerWith(Registrar registrar) {

        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_audio_recorder");
        channel.setMethodCallHandler(new VaisSpeechPlugin(registrar));

    }

    public VaisSpeechPlugin(Registrar registrar) {
        this.registrar = registrar;
        this.registrar.addRequestPermissionsResultListener(this);



    }

    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        final int REQUEST_RECORD_AUDIO_PERMISSION = 200;
        switch (requestCode) {
            case REQUEST_RECORD_AUDIO_PERMISSION:
                boolean granted = true;
                Log.d(LOG_NAME, "parsing result");
                for (int result : grantResults) {
                    if(result != PackageManager.PERMISSION_GRANTED) {
                        Log.d(LOG_NAME, "result" + result);
                        granted = false;
                    }
                }
                Log.d(LOG_NAME, "onRequestPermissionsResult -" + granted);
                if(_result != null) {
                    _result.success(granted);
                }
                return granted;
            default:
                Log.d(LOG_NAME, "onRequestPermissionsResult - false");
                return false;
        }
    }

    private boolean hasRecordPermission(){
        // if after [Marshmallow], we need to check permission on runtime
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
            return (ContextCompat.checkSelfPermission(registrar.context(), Manifest.permission.RECORD_AUDIO) == PackageManager.PERMISSION_GRANTED)
                    && (ContextCompat.checkSelfPermission(registrar.context(), Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED);
        } else {
            return ContextCompat.checkSelfPermission(registrar.context(), Manifest.permission.RECORD_AUDIO) == PackageManager.PERMISSION_GRANTED;
        }
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        // Log.d(LOG_NAME, "calling " + call.method);
        _result = result;

        switch (call.method){
            case "hasPermissions":
                handleHasPermission();
                break;
            case "init":
                handleInit(call, result);
                break;
            case "current":
                handleCurrent(call, result);
                break;
            case "start":
                handleStart(call, result);
                break;
            case "pause":
                handlePause(call, result);
                break;
            case "resume":
                handleResume(call, result);
                break;
            case "stop":
                handleStop(call, result);
                break;
            case "write":
                handleWrite(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void handleHasPermission(){
        if(hasRecordPermission()){
            Log.d(LOG_NAME, "handleHasPermission true");
            if(_result != null) {
                _result.success(true);
            }
        } else {
            Log.d(LOG_NAME, "handleHasPermission false");

            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
                ActivityCompat.requestPermissions(registrar.activity(), new String[]{Manifest.permission.RECORD_AUDIO, Manifest.permission.WRITE_EXTERNAL_STORAGE}, PERMISSIONS_REQUEST_RECORD_AUDIO);
            } else {
                ActivityCompat.requestPermissions(registrar.activity(), new String[]{Manifest.permission.RECORD_AUDIO}, PERMISSIONS_REQUEST_RECORD_AUDIO);
            }
        }

    }

    private void handleInit(MethodCall call, Result result)  {
        qe = new LinkedList();
        audioRecordService = new AudioRecordService();
        intent = new Intent(registrar.context(), audioRecordService.getClass());
        mSampleRate = Integer.parseInt(call.argument("sampleRate").toString());
        mFilePath = call.argument("path").toString();
        mExtension = call.argument("extension").toString();
        startForeground = (Boolean) call.argument("startForeground");
        isWriteFile = (Boolean) call.argument("isWriteFile");
        bufferSize = AudioRecord.getMinBufferSize(mSampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT);
        mStatus = "initialized";
        HashMap<String, Object> initResult = new HashMap<>();
        initResult.put("duration", 0);
        initResult.put("path", mFilePath);
        initResult.put("audioFormat", mExtension);
        initResult.put("peakPower", -120.00);
        initResult.put("averagePower", -120.00);
        initResult.put("isMeteringEnabled", true);
        initResult.put("status", mStatus);
        bindToService();
        result.success(initResult);
    }

    private void handleCurrent(MethodCall call, Result result) {
        HashMap<String, Object> current = qe.poll();
        if(current!=null){
            result.success(current);
        }
    }

    private void handleStart(MethodCall call, Result result) {
        LocalBroadcastManager.getInstance(registrar.activity()).registerReceiver(
                mMessageReceiver, new IntentFilter("GPSLocationUpdates"));
        mStatus = "recording";
        startServiceAndBind();
        try {
            mFileOutputStream = new FileOutputStream(getTempFilename());
        } catch (FileNotFoundException e) {
            result.error("", "cannot find the file", null);
            return;
        }
        result.success(null);
    }

    private void deleteTempFile() {
        File file = new File(getTempFilename());
        if(file.exists()) {
            file.delete();
        }
    }

    private String getTempFilename() {
        String filepath = mFilePath + ".temp";
        return filepath;
    }

    private BroadcastReceiver mMessageReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            // Get extra data included in the Intent
            HashMap<String, Object> hashMap = (HashMap<String, Object>) intent.getSerializableExtra("current");
            qe.add(hashMap);
            Log.d("kiemtra3",""+qe.size());
            if(qe.size() > 50){
                if(audioRecordService == null){
                    Log.d("kiemtra","audioRecordService null");
                }else{
//                    audioRecordService.stopForeground(true);
                    stopServiceAndUnBind();

                }
            }
        }
    };


    private void handleWrite(MethodCall call, Result result) {
        try {
            List<Integer> _byte =  (List<Integer>)call.argument("buffer");
            tempByteBuffer = new byte[_byte.size()];
            for (int i = 0; i < _byte.size(); i++) {
                tempByteBuffer[i] = (byte) _byte.get(i).intValue();
            }
            mFileOutputStream.write(tempByteBuffer);
        } catch (IOException e) {
            e.printStackTrace();
        }
        result.success(null);
    }


    private void handlePause(MethodCall call, Result result) {
        mStatus = "paused";
        mAudioRecordService.pauseRecord();

        qe.clear();
        result.success(null);
    }

    private void handleResume(MethodCall call, Result result) {
        mStatus = "recording";
        mAudioRecordService.resumeRecord();

        result.success(null);
    }

    private void handleStop(MethodCall call, Result result) {
        if(mStatus.equals("stopped")) {
            result.success(null);
        } else {
            mStatus = "stopped";
            LocalBroadcastManager.getInstance(registrar.context()).unregisterReceiver(mMessageReceiver);

            // Return Recording Object
            HashMap<String, Object> currentResult = new HashMap<>();
            currentResult.put("duration", 0);
            currentResult.put("path", mFilePath);
            currentResult.put("audioFormat", mExtension);
            currentResult.put("peakPower", -120.00);
            currentResult.put("averagePower", -120.00);
            currentResult.put("isMeteringEnabled", true);
            currentResult.put("status", mStatus);
            currentResult.put("buffer", tempByteBuffer);


            qe.clear();
            stopServiceAndUnBind();
            copyWaveFile(getTempFilename(), mFilePath);
            deleteTempFile();
            result.success(currentResult);


        }

    }
    private void copyWaveFile(String inFilename, String outFilename) {
        FileInputStream in = null;
        FileOutputStream out = null;
        long totalAudioLen = 0;
        long totalDataLen = totalAudioLen + 36;
        long longSampleRate = mSampleRate;
        int channels = 1;
        long byteRate = RECORDER_BPP * mSampleRate * channels / 8;

        byte[] data = new byte[bufferSize];

        try {
            in = new FileInputStream(inFilename);
            out = new FileOutputStream(outFilename);
            totalAudioLen = in.getChannel().size();
            totalDataLen = totalAudioLen + 36;

            WriteWaveFileHeader(out, totalAudioLen, totalDataLen,
                    longSampleRate, channels, byteRate);

            while (in.read(data) != -1) {
                out.write(data);
            }

            in.close();
            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void WriteWaveFileHeader(FileOutputStream out, long totalAudioLen,
                                     long totalDataLen, long longSampleRate, int channels, long byteRate)
            throws IOException {
        byte[] header = new byte[44];

        header[0] = 'R'; // RIFF/WAVE header
        header[1] = 'I';
        header[2] = 'F';
        header[3] = 'F';
        header[4] = (byte) (totalDataLen & 0xff);
        header[5] = (byte) ((totalDataLen >> 8) & 0xff);
        header[6] = (byte) ((totalDataLen >> 16) & 0xff);
        header[7] = (byte) ((totalDataLen >> 24) & 0xff);
        header[8] = 'W';
        header[9] = 'A';
        header[10] = 'V';
        header[11] = 'E';
        header[12] = 'f'; // 'fmt ' chunk
        header[13] = 'm';
        header[14] = 't';
        header[15] = ' ';
        header[16] = 16; // 4 bytes: size of 'fmt ' chunk
        header[17] = 0;
        header[18] = 0;
        header[19] = 0;
        header[20] = 1; // format = 1
        header[21] = 0;
        header[22] = (byte) channels;
        header[23] = 0;
        header[24] = (byte) (longSampleRate & 0xff);
        header[25] = (byte) ((longSampleRate >> 8) & 0xff);
        header[26] = (byte) ((longSampleRate >> 16) & 0xff);
        header[27] = (byte) ((longSampleRate >> 24) & 0xff);
        header[28] = (byte) (byteRate & 0xff);
        header[29] = (byte) ((byteRate >> 8) & 0xff);
        header[30] = (byte) ((byteRate >> 16) & 0xff);
        header[31] = (byte) ((byteRate >> 24) & 0xff);
        header[32] = (byte) (1); // block align
        header[33] = 0;
        header[34] = RECORDER_BPP; // bits per sample
        header[35] = 0;
        header[36] = 'd';
        header[37] = 'a';
        header[38] = 't';
        header[39] = 'a';
        header[40] = (byte) (totalAudioLen & 0xff);
        header[41] = (byte) ((totalAudioLen >> 8) & 0xff);
        header[42] = (byte) ((totalAudioLen >> 16) & 0xff);
        header[43] = (byte) ((totalAudioLen >> 24) & 0xff);

        out.write(header, 0, 44);
    }















    private boolean mIsServiceBound = false;
    private AudioRecordService mAudioRecordService;

    private final ServiceConnection serviceConnection = new ServiceConnection() {
        @Override public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
            mIsServiceBound = true;
            mAudioRecordService =
                    ((AudioRecordService.ServiceBinder) iBinder).getService();
            Log.i("Tesing", " " + mAudioRecordService.isRecording() + " recording");
        }

        @Override public void onServiceDisconnected(ComponentName componentName) {
        }
    };

    public void startServiceAndBind() {
        Log.d("kiemtra","startServiceAndBind");
        intent.putExtra("mFilePath",mFilePath);
        intent.putExtra("mSampleRate",mSampleRate);
        intent.putExtra("startForeground",startForeground);
        intent.putExtra("isWriteFile",isWriteFile);

        registrar.context().startService(intent);
        bindToService();
    }


    public void stopServiceAndUnBind() {
        registrar.context().stopService(intent);
        unbindFromService();
    }

    void unbindFromService() {
        unRegisterLocalBroadCastReceiver();
        if (mIsServiceBound) {
            try{
                mIsServiceBound = false;
                registrar.context().unbindService(serviceConnection);
                audioRecordService.stopSelf();
                audioRecordService.stopForeground(true);
            }catch (Exception e){
                audioRecordService.stopSelf();
                Log.e("kiemtra",e.getMessage());
            }

        }
    }
    private void unRegisterLocalBroadCastReceiver() {
        LocalBroadcastManager.getInstance(registrar.context()).unregisterReceiver(serviceUpdateReceiver);
    }
    void bindToService() {
        Log.d("kiemtra","bindToService");
        registrar.context().bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);
        registerLocalBroadCastReceiver();
    }
    private void registerLocalBroadCastReceiver() {
        LocalBroadcastManager.getInstance(registrar.context())
                .registerReceiver(serviceUpdateReceiver, new IntentFilter(AppConstants.ACTION_IN_SERVICE));
    }
    private final BroadcastReceiver serviceUpdateReceiver = new BroadcastReceiver() {
        @Override public void onReceive(Context context, Intent intent) {
            if (!intent.hasExtra(AppConstants.ACTION_IN_SERVICE)) return;
            String actionExtra = intent.getStringExtra(AppConstants.ACTION_IN_SERVICE);
        }
    };


}

